use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.vme64x_pkg.all;
use work.vme64x_sim_pkg.all;

entity tb_vmecore_test is
end tb_vmecore_test;


architecture behav of tb_vmecore_test is
  signal rst_n_i             : std_logic;
  signal clk_20m_vcxo_i      : std_logic := '0';
  signal vme_write_n_i       : std_logic;
  signal vme_sysreset_n_i    : std_logic;
  signal vme_retry_oe_o      : std_logic;
  signal vme_retry_n_o       : std_logic;
  signal vme_lword_n_b       : std_logic;
  signal vme_iackout_n_o     : std_logic;
  signal vme_iackin_n_i      : std_logic;
  signal vme_iack_n_i        : std_logic;
  signal vme_gap_i           : std_logic;
  signal vme_dtack_oe_o      : std_logic;
  signal vme_dtack_n_o       : std_logic;
  signal vme_ds_n_i          : std_logic_vector(1 downto 0);
  signal vme_data_oe_n_o     : std_logic;
  signal vme_data_dir_o      : std_logic;
  signal vme_berr_o          : std_logic;
  signal vme_as_n_i          : std_logic;
  signal vme_addr_oe_n_o     : std_logic;
  signal vme_addr_dir_o      : std_logic;
  signal vme_irq_o           : std_logic_vector(7 downto 1);
  signal vme_ga_i            : std_logic_vector(4 downto 0);
  signal vme_data_b          : std_logic_vector(31 downto 0);
  signal vme_am_i            : std_logic_vector(5 downto 0);
  signal vme_addr_b          : std_logic_vector(31 downto 1);
  signal pcbrev_i            : std_logic_vector(4 downto 0);
  signal fp_led_line_oen_o   : std_logic_vector(1 downto 0);
  signal fp_led_line_o       : std_logic_vector(1 downto 0);
  signal fp_led_column_o     : std_logic_vector(3 downto 0);
  signal fp_gpio1_b          : std_logic;
  signal fp_gpio2_b          : std_logic;
  signal fp_gpio3_b          : std_logic;
  signal fp_gpio4_b          : std_logic;
  signal fp_term_en_o        : std_logic_vector(4 downto 1);
  signal fp_gpio1_a2b_o      : std_logic;
  signal fp_gpio2_a2b_o      : std_logic;
  signal fp_gpio34_a2b_o     : std_logic;

  signal vme_in : t_vme64x_in;
  signal vme_out : t_vme64x_out;
  signal vme_timeout : boolean := false;

  signal stop : boolean := false;
begin
  --  Clock.
  process
  begin
    clk_20m_vcxo_i <= '0';
    wait for 25 ns;

    clk_20m_vcxo_i <= '1';
    wait for 25 ns;

    if stop then
      wait;
    end if;
  end process;

  process
  is
    procedure dump32 (base : natural; nword : natural)
    is
      variable addr : std_logic_vector (31 downto 0);
      variable d32 : lword_t;
    begin
      for i in 0 to nword - 1 loop
        addr := std_logic_vector(to_unsigned(base + i * 4, 32));
        read32 (vme_in, vme_out, vme_timeout, x"00_10_00_00" or addr,
                c_AM_A24, d32);
        write(output, hex(addr) & ": " & hex(d32) & LF);
      end loop;
    end dump32;

    variable d8 : byte_t;
    variable d16 : word_t;
    variable d32 : lword_t;

    variable arr64 : qword_array_t (31 downto 0);
  begin
    --  Each scenario starts with a reset.
    --  VME reset
    report "VME reset";
    vme_in.ga <= '1' & slave_ga;
    rst_n_i <= '0';
    VME_in.RST_n <= '0';
    VME_in.AS_n <= '1';
    VME_in.ADDR <= (others => '1');
    VME_in.AM <= (others => '1');
    VME_in.DS_n <= "11";
    VME_in.iackin_n <= '1';
    wait for 40 ns;
    VME_in.RST_n <= '1';
    rst_n_i <= '1';
    wait for 40 ns;

    report "Set ADER";

    -- Set ADER
    write8_conf (vme_in, vme_out, vme_timeout, x"7_ff77", x"10");
    write8_conf (vme_in, vme_out, vme_timeout, x"7_ff7f", c_AM_2EVME_6U & "00");
    read8_conf  (vme_in, vme_out, vme_timeout, x"7_ff77", d8);
    assert d8 = x"10" report "bad ADER0 value" severity error;

    report "Set ENable";

    -- Enable card
    write8_conf (vme_in, vme_out, vme_timeout, x"7_fffb", b"0001_0000");
    read8_conf  (vme_in, vme_out, vme_timeout, x"7_fffb", d8);
    assert d8 = b"0001_0000" report "module must be enabled"
      severity error;
    report "read data: " & hex (d8);

    --  2eSST transfer at 267
    read64_2esst (vme_in, vme_out, vme_timeout, x"00_10_00_00",
                  c_XAM_A32_2ESST, "0001", arr64);
    for i in arr64'range loop
      write(output, hex(arr64(i)) & LF);
    end loop;

    -- dump_CR (vme_in, vme_out, vme_timeout);

    report "Dump patterns";
    dump32 (16#c000#, 4);

    if false then
      report "VME CR";
      dump_CR (vme_in, vme_out, vme_timeout);
    end if;

    report "done";
    stop <= true;
    wait;
  end process;

  vme_data_b <= vme_in.data when not (vme_out.data_oe_n = '0'
                                      and vme_out.data_dir = '1')
                else (others => 'H');
  vme_out.data <= vme_data_b;

  vme_addr_b <= vme_in.addr when not (vme_out.addr_oe_n = '0'
                                      and vme_out.addr_dir = '1')
                else (others => 'H');
  vme_out.addr <= vme_addr_b;

  vme_lword_n_b <= vme_in.lword_n when not (vme_out.addr_oe_n = '0'
                                      and vme_out.addr_dir = '1')
                   else 'H';
  vme_out.lword_n <= vme_lword_n_b;

  --  ga is inverted.
  vme_ga_i <= not vme_in.ga (4 downto 0);
  vme_gap_i <= vme_in.ga(5);

  vme_out.berr_n <= not vme_berr_o;

  dut: entity work.svec_vmecore_test_top
    generic map (
      g_SIMULATION  => True
      )
    port map (
      rst_n_i             => rst_n_i,
      clk_20m_vcxo_i      => clk_20m_vcxo_i,

      vme_gap_i           => vme_gap_i,
      vme_ga_i            => vme_ga_i,

      vme_as_n_i          => vme_in.as_n,
      vme_sysreset_n_i    => vme_in.rst_n,
      vme_write_n_i       => vme_in.write_n,
      vme_am_i            => vme_in.am,
      vme_ds_n_i          => vme_in.ds_n,
      vme_iack_n_i        => vme_in.iack_n,
      vme_iackin_n_i      => vme_in.iackin_n,

      vme_iackout_n_o     => vme_out.iackout_n,
      vme_dtack_n_o       => vme_out.dtack_n,
      vme_dtack_oe_o      => vme_out.dtack_oe,
      vme_data_dir_o      => vme_out.data_dir,
      vme_data_oe_n_o     => vme_out.data_oe_n,
      vme_addr_dir_o      => vme_out.addr_dir,
      vme_addr_oe_n_o     => vme_out.addr_oe_n,
      vme_retry_n_o       => vme_out.retry_n,
      vme_retry_oe_o      => vme_out.retry_oe,

      vme_berr_o          => vme_berr_o,
      vme_irq_o           => vme_irq_o,

      vme_lword_n_b       => vme_lword_n_b,
      vme_data_b          => vme_data_b,
      vme_addr_b          => vme_addr_b,

      fp_led_line_oen_o   => fp_led_line_oen_o,
      fp_led_line_o       => fp_led_line_o,
      fp_led_column_o     => fp_led_column_o);
end behav;
