-- Do not edit.  Generated on Tue Apr 07 14:01:51 2020 by tgingold
-- With Cheby 1.4.dev0 and these options:
--  -i tester_top_regs.cheby --gen-hdl


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.wishbone_pkg.all;

entity tester_top_regs is
  port (
    rst_n_i              : in    std_logic;
    clk_i                : in    std_logic;
    wb_i                 : in    t_wishbone_slave_in;
    wb_o                 : out   t_wishbone_slave_out;

    -- General status
    -- Set when WR has time
    status_time_valid_i  : in    std_logic;
    -- Set when WR has link
    status_link_valid_i  : in    std_logic;

    -- TAI from wr
    tm_tai_i             : in    std_logic_vector(63 downto 0);

    -- number of cycles (16Mhz) within the second
    tm_cycles_i          : in    std_logic_vector(31 downto 0);

    -- absolute TAI (LSB) for interrupts
    start_tai_o          : out   std_logic_vector(31 downto 0);

    -- absolute cycles for interrupts
    start_cycles_o       : out   std_logic_vector(31 downto 0);

    -- number of cycles between interrupts
    period_cycles_o      : out   std_logic_vector(31 downto 0);

    -- number of interrupts to deliver
    total_count_o        : out   std_logic_vector(31 downto 0);

    -- number of interrupts delivered.
    current_count_i      : in    std_logic_vector(31 downto 0);

    -- the current number of interrupts must be written to ack the interrupt.
    ack_int_o            : out   std_logic_vector(31 downto 0);
    ack_int_wr_o         : out   std_logic;

    -- number of interrupts successfully ack-ed; cleared at the start
    ack_count_i          : in    std_logic_vector(31 downto 0)
  );
end tester_top_regs;

architecture syn of tester_top_regs is
  signal adr_int                        : std_logic_vector(13 downto 2);
  signal rd_req_int                     : std_logic;
  signal wr_req_int                     : std_logic;
  signal rd_ack_int                     : std_logic;
  signal wr_ack_int                     : std_logic;
  signal wb_en                          : std_logic;
  signal ack_int                        : std_logic;
  signal wb_rip                         : std_logic;
  signal wb_wip                         : std_logic;
  signal start_tai_reg                  : std_logic_vector(31 downto 0);
  signal start_tai_wreq                 : std_logic;
  signal start_tai_wack                 : std_logic;
  signal start_cycles_reg               : std_logic_vector(31 downto 0);
  signal start_cycles_wreq              : std_logic;
  signal start_cycles_wack              : std_logic;
  signal period_cycles_reg              : std_logic_vector(31 downto 0);
  signal period_cycles_wreq             : std_logic;
  signal period_cycles_wack             : std_logic;
  signal total_count_reg                : std_logic_vector(31 downto 0);
  signal total_count_wreq               : std_logic;
  signal total_count_wack               : std_logic;
  signal ack_int_reg                    : std_logic_vector(31 downto 0);
  signal ack_int_wreq                   : std_logic;
  signal ack_int_wack                   : std_logic;
  signal rd_ack_d0                      : std_logic;
  signal rd_dat_d0                      : std_logic_vector(31 downto 0);
  signal wr_req_d0                      : std_logic;
  signal wr_adr_d0                      : std_logic_vector(13 downto 2);
  signal wr_dat_d0                      : std_logic_vector(31 downto 0);
  signal wr_sel_d0                      : std_logic_vector(3 downto 0);
begin

  -- WB decode signals
  adr_int <= wb_i.adr(13 downto 2);
  wb_en <= wb_i.cyc and wb_i.stb;

  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        wb_rip <= '0';
      else
        wb_rip <= (wb_rip or (wb_en and not wb_i.we)) and not rd_ack_int;
      end if;
    end if;
  end process;
  rd_req_int <= (wb_en and not wb_i.we) and not wb_rip;

  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        wb_wip <= '0';
      else
        wb_wip <= (wb_wip or (wb_en and wb_i.we)) and not wr_ack_int;
      end if;
    end if;
  end process;
  wr_req_int <= (wb_en and wb_i.we) and not wb_wip;

  ack_int <= rd_ack_int or wr_ack_int;
  wb_o.ack <= ack_int;
  wb_o.stall <= not ack_int and wb_en;
  wb_o.rty <= '0';
  wb_o.err <= '0';

  -- pipelining for wr-in+rd-out
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        rd_ack_int <= '0';
        wr_req_d0 <= '0';
      else
        rd_ack_int <= rd_ack_d0;
        wb_o.dat <= rd_dat_d0;
        wr_req_d0 <= wr_req_int;
        wr_adr_d0 <= adr_int;
        wr_dat_d0 <= wb_i.dat;
        wr_sel_d0 <= wb_i.sel;
      end if;
    end if;
  end process;

  -- Register vendor_id

  -- Register device_id

  -- Register version

  -- Register bom

  -- Register source_id0

  -- Register source_id1

  -- Register source_id2

  -- Register source_id3

  -- Register status

  -- Register tm_tai

  -- Register tm_cycles

  -- Register start_tai
  start_tai_o <= start_tai_reg;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        start_tai_reg <= "00000000000000000000000000000000";
        start_tai_wack <= '0';
      else
        if start_tai_wreq = '1' then
          start_tai_reg <= wr_dat_d0;
        end if;
        start_tai_wack <= start_tai_wreq;
      end if;
    end if;
  end process;

  -- Register start_cycles
  start_cycles_o <= start_cycles_reg;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        start_cycles_reg <= "00000000000000000000000000000000";
        start_cycles_wack <= '0';
      else
        if start_cycles_wreq = '1' then
          start_cycles_reg <= wr_dat_d0;
        end if;
        start_cycles_wack <= start_cycles_wreq;
      end if;
    end if;
  end process;

  -- Register period_cycles
  period_cycles_o <= period_cycles_reg;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        period_cycles_reg <= "00000000000000000000000000000000";
        period_cycles_wack <= '0';
      else
        if period_cycles_wreq = '1' then
          period_cycles_reg <= wr_dat_d0;
        end if;
        period_cycles_wack <= period_cycles_wreq;
      end if;
    end if;
  end process;

  -- Register total_count
  total_count_o <= total_count_reg;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        total_count_reg <= "00000000000000000000000000000000";
        total_count_wack <= '0';
      else
        if total_count_wreq = '1' then
          total_count_reg <= wr_dat_d0;
        end if;
        total_count_wack <= total_count_wreq;
      end if;
    end if;
  end process;

  -- Register current_count

  -- Register ack_int
  ack_int_o <= ack_int_reg;
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        ack_int_reg <= "00000000000000000000000000000000";
        ack_int_wack <= '0';
      else
        if ack_int_wreq = '1' then
          ack_int_reg <= wr_dat_d0;
        end if;
        ack_int_wack <= ack_int_wreq;
      end if;
    end if;
  end process;
  ack_int_wr_o <= ack_int_wack;

  -- Register ack_count

  -- Process for write requests.
  process (wr_adr_d0, wr_req_d0, start_tai_wack, start_cycles_wack, period_cycles_wack, total_count_wack, ack_int_wack) begin
    start_tai_wreq <= '0';
    start_cycles_wreq <= '0';
    period_cycles_wreq <= '0';
    total_count_wreq <= '0';
    ack_int_wreq <= '0';
    case wr_adr_d0(13 downto 3) is
    when "10000000000" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg vendor_id
        wr_ack_int <= wr_req_d0;
      when "1" => 
        -- Reg device_id
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000000001" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg version
        wr_ack_int <= wr_req_d0;
      when "1" => 
        -- Reg bom
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000000010" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg source_id0
        wr_ack_int <= wr_req_d0;
      when "1" => 
        -- Reg source_id1
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000000011" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg source_id2
        wr_ack_int <= wr_req_d0;
      when "1" => 
        -- Reg source_id3
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000000100" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg status
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000000101" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg tm_tai
        wr_ack_int <= wr_req_d0;
      when "1" => 
        -- Reg tm_tai
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000000110" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg tm_cycles
        wr_ack_int <= wr_req_d0;
      when "1" => 
        -- Reg start_tai
        start_tai_wreq <= wr_req_d0;
        wr_ack_int <= start_tai_wack;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000000111" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg start_cycles
        start_cycles_wreq <= wr_req_d0;
        wr_ack_int <= start_cycles_wack;
      when "1" => 
        -- Reg period_cycles
        period_cycles_wreq <= wr_req_d0;
        wr_ack_int <= period_cycles_wack;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000001000" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg total_count
        total_count_wreq <= wr_req_d0;
        wr_ack_int <= total_count_wack;
      when "1" => 
        -- Reg current_count
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when "10000001001" => 
      case wr_adr_d0(2 downto 2) is
      when "0" => 
        -- Reg ack_int
        ack_int_wreq <= wr_req_d0;
        wr_ack_int <= ack_int_wack;
      when "1" => 
        -- Reg ack_count
        wr_ack_int <= wr_req_d0;
      when others =>
        wr_ack_int <= wr_req_d0;
      end case;
    when others =>
      wr_ack_int <= wr_req_d0;
    end case;
  end process;

  -- Process for read requests.
  process (adr_int, rd_req_int, status_time_valid_i, status_link_valid_i, tm_tai_i, tm_cycles_i, start_tai_reg, start_cycles_reg, period_cycles_reg, total_count_reg, current_count_i, ack_count_i) begin
    -- By default ack read requests
    rd_dat_d0 <= (others => 'X');
    case adr_int(13 downto 3) is
    when "10000000000" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg vendor_id
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "00000000000000000001000011011100";
      when "1" => 
        -- Reg device_id
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "01010100010100110101010000000001";
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000000001" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg version
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "00000000000000000000000000000001";
      when "1" => 
        -- Reg bom
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "11111111111111100000000000000000";
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000000010" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg source_id0
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "00000000000000000000000000000000";
      when "1" => 
        -- Reg source_id1
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "00000000000000000000000000000000";
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000000011" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg source_id2
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "00000000000000000000000000000000";
      when "1" => 
        -- Reg source_id3
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= "00000000000000000000000000000000";
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000000100" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg status
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0(0) <= status_time_valid_i;
        rd_dat_d0(1) <= status_link_valid_i;
        rd_dat_d0(31 downto 2) <= (others => '0');
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000000101" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg tm_tai
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= tm_tai_i(63 downto 32);
      when "1" => 
        -- Reg tm_tai
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= tm_tai_i(31 downto 0);
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000000110" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg tm_cycles
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= tm_cycles_i;
      when "1" => 
        -- Reg start_tai
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= start_tai_reg;
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000000111" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg start_cycles
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= start_cycles_reg;
      when "1" => 
        -- Reg period_cycles
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= period_cycles_reg;
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000001000" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg total_count
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= total_count_reg;
      when "1" => 
        -- Reg current_count
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= current_count_i;
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when "10000001001" => 
      case adr_int(2 downto 2) is
      when "0" => 
        -- Reg ack_int
        rd_ack_d0 <= rd_req_int;
      when "1" => 
        -- Reg ack_count
        rd_ack_d0 <= rd_req_int;
        rd_dat_d0 <= ack_count_i;
      when others =>
        rd_ack_d0 <= rd_req_int;
      end case;
    when others =>
      rd_ack_d0 <= rd_req_int;
    end case;
  end process;
end syn;
